@@ -1,11 +1,14 @@
+from __future__ import absolute_import
+import sys
 import logging
 from time import time
 from types import MethodType
 from datetime import datetime
 
-from . import loader
-from .queue import Queue
-from .channel import LazyChannel
+from kuyruk import importer
+from kuyruk.queue import Queue
+from kuyruk.channel import LazyChannel
+from kuyruk.eventmixin import EventMixin
 
 logger = logging.getLogger(__name__)
 
@@ -20,10 +23,11 @@ def profile(f):
     return inner
 
 
-class Task(object):
+class Task(EventMixin):
 
     def __init__(self, f, kuyruk, queue='kuyruk',
                  local=False, eager=False, retry=0):
+        super(Task, self).__init__()
         self.f = f
         self.kuyruk = kuyruk
         self.queue = queue
@@ -31,8 +35,6 @@ class Task(object):
         self.eager = eager
         self.retry = retry
         self.cls = None
-        self.before_task_functions = []
-        self.after_task_functions = []
 
     def __repr__(self):
         return "<Task of %r>" % self.name
@@ -93,14 +95,23 @@ class Task(object):
 
     @profile
     def run(self, args, kwargs):
-        """Run the wrapped function with before and after task functions."""
-        def run(functions):
-            [f(self, args, kwargs) for f in functions]
+        """Run the wrapped function and event handlers."""
+        def run(functions, **extra):
+            for f in functions:
+                f(self, args, kwargs, **extra)
 
         try:
             run(self.kuyruk.before_task_functions)
             run(self.before_task_functions)
-            self.f(*args, **kwargs)  # call wrapped function
+            return_value = self.f(*args, **kwargs)  # call wrapped function
+        except Exception:
+            exc_info = sys.exc_info()
+            run(self.on_exception_functions, exc_info=exc_info)
+            run(self.kuyruk.on_exception_functions, exc_info=exc_info)
+            raise
+        else:
+            run(self.after_return_functions, return_value=return_value)
+            run(self.kuyruk.after_return_functions, return_value=return_value)
         finally:
             run(self.after_task_functions)
             run(self.kuyruk.after_task_functions)
@@ -117,7 +128,7 @@ class Task(object):
     def module_name(self):
         name = self.f.__module__
         if name == '__main__':
-            name = loader.get_main_module().name
+            name = importer.get_main_module().name
         return name
 
     @property
@@ -125,14 +136,6 @@ class Task(object):
         if self.cls:
             return self.cls.__name__
 
-    def before_task(self, f):
-        self.before_task_functions.append(f)
-        return f
-
-    def after_task(self, f):
-        self.after_task_functions.append(f)
-        return f
-
 
 class TaskResult(object):
 
