@@ -1,16 +1,14 @@
 from __future__ import absolute_import
 import os
+import socket
 import signal
 import logging
 import traceback
-import threading
-import multiprocessing
 from time import sleep
-
 from setproctitle import setproctitle
-
 from kuyruk import importer
 from kuyruk.queue import Queue
+from kuyruk.process import Process
 from kuyruk.channel import LazyChannel
 from kuyruk.consumer import Consumer
 from kuyruk.helpers import start_daemon_thread
@@ -18,15 +16,14 @@ from kuyruk.helpers import start_daemon_thread
 logger = logging.getLogger(__name__)
 
 
-class Worker(multiprocessing.Process):
+class Worker(Process):
 
     def __init__(self, queue_name, config):
         """
         :param queue_name: Queue name that this worker gets the messages from
         :param config: Configuration object
         """
-        super(Worker, self).__init__()
-        self.config = config
+        super(Worker, self).__init__(config)
         self.channel = LazyChannel(
             self.config.RABBIT_HOST, self.config.RABBIT_PORT,
             self.config.RABBIT_USER, self.config.RABBIT_PASSWORD)
@@ -34,7 +31,6 @@ class Worker(multiprocessing.Process):
         is_local = queue_name.startswith('@')
         self.queue = Queue(queue_name, self.channel, local=is_local)
         self.consumer = Consumer(self.queue)
-        self.shutdown_pending = threading.Event()
 
     def run(self):
         """Run worker until stop flag is set.
@@ -43,10 +39,8 @@ class Worker(multiprocessing.Process):
         it will be invoked when worker.start() is called.
 
         """
-        logger.debug('Process id: %s', os.getpid())
-        logger.debug('Process group id: %s', os.getpgrp())
+        super(Worker, self).run()
         setproctitle("kuyruk: worker on %s" % self.queue_name)
-        self.register_signals()
         self.queue.declare()
         self.channel.basic_qos(prefetch_count=1)
         self.channel.tx_select()
@@ -56,6 +50,7 @@ class Worker(multiprocessing.Process):
         start_daemon_thread(self.watch_load)
         if self.config.MAX_RUN_TIME > 0:
             start_daemon_thread(self.shutdown_timer)
+        self.maybe_start_manager_thread()
 
         # Consume messages
         with self.consumer.consume() as messages:
@@ -118,9 +113,10 @@ class Worker(multiprocessing.Process):
             task_description['object_id'],
             task_description['args'],
             task_description['kwargs'])
-        task, cls = importer.import_task(module, cls, function,
-                                       self.config.IMPORT_PATH)
+        task, cls = importer.import_task(
+            module, cls, function, self.config.IMPORT_PATH)
 
+        # Fetch object and prepent to args if class task
         if cls:
             obj = cls.get(object_id)
             if not obj:
@@ -148,7 +144,7 @@ class Worker(multiprocessing.Process):
         while not self.shutdown_pending.is_set():
             if not self.is_master_alive():
                 logger.critical('Master is dead')
-                self.shutdown()
+                self.warm_shutdown()
             sleep(1)
 
     def watch_load(self):
@@ -167,25 +163,41 @@ class Worker(multiprocessing.Process):
         """
         sleep(self.config.MAX_RUN_TIME)
         logger.warning('Run time reached zero, cancelling consume.')
-        self.shutdown()
+        self.warm_shutdown()
 
     def register_signals(self):
         # SIGINT is ignored because when pressed Ctrl-C
-        # SIGINT sent to both master and workers while.
+        # it is sent to both master and workers.
         signal.signal(signal.SIGINT, signal.SIG_IGN)
-        signal.signal(signal.SIGTERM, self.sigterm_handler)
+        signal.signal(signal.SIGTERM, self.handle_sigterm)
         signal.signal(signal.SIGUSR1, print_stack)  # for debugging
 
-    def sigterm_handler(self, signum, frame):
+    def handle_sigterm(self, signum, frame):
         logger.warning("Catched SIGTERM")
-        self.shutdown()
+        self.warm_shutdown()
 
-    def shutdown(self):
+    def warm_shutdown(self):
         """Shutdown gracefully."""
         logger.warning("Shutting down worker gracefully")
         self.shutdown_pending.set()
         self.consumer.stop()
 
+    def get_stats(self):
+        """Generate stats to be sent to manager."""
+        method = self.queue.declare().method
+        return {
+            'type': 'worker',
+            'hostname': socket.gethostname(),
+            'uptime': self.uptime,
+            'pid': os.getpid(),
+            'ppid': os.getppid(),
+            'queue': {
+                'name': method.queue,
+                'messages_ready': method.message_count,
+                'consumers': method.consumer_count,
+            }
+        }
+
 
 def print_stack(sig, frame):
     print '=' * 70
